
import java.awt.*;
import javax.swing.*;
import javax.media.opengl.*;

import com.sun.opengl.util.GLUT;

/**
 * A Jogl/OpenGL application that just draws a set of axes in
 * 3-space, Each axis represented as a cylinder topped by a cone.
 */
public class Axes3D extends JPanel implements GLEventListener {

   public static void main(String[] args) {
      JFrame window = new JFrame("3D axes");
      window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      window.setContentPane(new Axes3D());
      window.pack();
      window.setVisible(true);
   }
   
   private GLUT glut;  // For access some drawing routines from the
                       // OpenGL Utilties Toolkit.  glut is used here
                       // to draw cylinders and cones.
   
   /**
    * Make a regular JPanel to hold the GLJPanel (although in this
    * program, that's all it holds.
    */
   public Axes3D() {
      GLJPanel drawable = new GLJPanel();
      drawable.setPreferredSize(new Dimension(600,600));
      drawable.addGLEventListener(this);
      setLayout(new BorderLayout());
      add(drawable, BorderLayout.CENTER);
   }

   /**
    * This method will be called when the GLJPanel is first
    * created.  It can be used to initialize the GL context.
    */
   public void init(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glEnable(GL.GL_LIGHTING);
      gl.glEnable(GL.GL_LIGHT0);
      gl.glEnable(GL.GL_DEPTH_TEST);
      gl.glShadeModel(GL.GL_SMOOTH);
      glut = new GLUT();
   }

   /**
    * Draw a set of coordinate axes, 
    */
   public void display(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
      gl.glLoadIdentity();
      
      /* Rotate the whole set of coordinate axes for a better view. */

      gl.glRotatef(15,1,0,0);
      gl.glRotatef(-35,0,1,0);
      
      /* Draw the x-axis in red. */
      
      gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE,
                new float[] { 1, 0, 0, 1 }, 0);
      gl.glPushMatrix();
      gl.glRotatef(90,0,1,0);  // drawAxis draws a z-axis; rotate it onto the x-axis.
      drawAxis(gl);
      gl.glPopMatrix();
      
      /* Draw the y-axis in green. */
      
      gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE,
                new float[] { 0, 0.8f, 0, 1 }, 0);  // Not quite full intensity green.
      gl.glPushMatrix();
      gl.glRotatef(-90,1,0,0);  // drawAxis draws a z-axis; rotate it onto the y-axis.
      drawAxis(gl);
      gl.glPopMatrix();
      
      /* Draw the z-axis in blue. */
      
      gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE,
                new float[] { 0.4f, 0.4f, 1, 1 }, 0);  // Somewhat unsaturated blue.
      drawAxis(gl);
   }
   
   
   /**
    * Draws a coordinate axis along the positive z-axis that consists of a narrow
    * cylinder of length 1 topped by a cone of height 0.3.
    */
   private void drawAxis(GL gl) {
      gl.glPushMatrix();
      glut.glutSolidCylinder(0.02, 1, 6, 10); // Cylinder, radius 0.02, height 1,
                                              // base at (0,0,0), lying on z-axis.
      gl.glTranslatef(0,0,1); // Move the cone to the top of the cylinder
      glut.glutSolidCone(0.1, 0.3, 12, 5); // Cone, radius 0.1, height 0.3,
                                           // base at (0,0,0), pointing along z-axis.
      gl.glPopMatrix();
   }

   public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
      GL gl = drawable.getGL();
      double s = 1.5;  // limits of cube that we want to view go from -s to s.
      gl.glMatrixMode(GL.GL_PROJECTION);
      gl.glLoadIdentity();  // Start with the identity transform.
      if (width > height) {  // Expand x limits to match viewport aspect ratio.
         double ratio = (double)width/height;
         gl.glOrtho(-s*ratio, s*ratio, -s, s, -s, s);
      }
      else {  // Expand y limits to match viewport aspect ratio.
         double ratio = (double)height/width;
         gl.glOrtho(-s, s, -s*ratio, s*ratio, -s, s);
      }
      gl.glMatrixMode(GL.GL_MODELVIEW);
   }

   public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
         boolean deviceChanged) {
   }
}
