
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.opengl.*;

/**
 * A very simple OpenGL animation, using a Timer to drive the
 * animation and a rotation transform to rotate a picture.
 */
public class BasicJoglAnimation2D extends JPanel implements GLEventListener {

   public static void main(String[] args) {
      JFrame window = new JFrame("Basic JOGL Animation 2D");
      window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      window.setContentPane(new BasicJoglAnimation2D());
      window.pack();
      window.setVisible(true);
   }
   
   private GLJPanel drawable;  // The OpenGL display panel.
   private int frameNumber;    // Current frame number, increases by 1 in each frame.
   
   public BasicJoglAnimation2D() {
      drawable = new GLJPanel();
      drawable.setPreferredSize(new Dimension(600,600));
      drawable.addGLEventListener(this);
      setLayout(new BorderLayout());
      add(drawable, BorderLayout.CENTER);
      Timer timer = new Timer(30, new ActionListener() {
         public void actionPerformed(ActionEvent evt) {
            frameNumber++;
            drawable.repaint();
         }
      });
      timer.setInitialDelay(1000);
      timer.start();
   }

   /**
    * This method is called when the GLJPanel is created.  It initializes
    * the GL context.  Here, it sets the clear color to be light blue.
    */
   public void init(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glClearColor(0.8f, 0.8f, 1, 1);
   }

   /**
    * This method is called when the GLJPanel needs to be redrawn.  Here,
    * a yellow triangle, outlined in black, is drawn with a rotation 
    * equal to the frame number (measured in degrees).
    */
   public void display(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glClear(GL.GL_COLOR_BUFFER_BIT);
      gl.glLoadIdentity();
      gl.glRotated(frameNumber, 0, 0, 1);  // Rotation to be applied to the triangle.
      gl.glColor3f(1.0f, 1.0f, 0.0f);
      gl.glBegin(GL.GL_POLYGON);
      gl.glVertex2f(-0.5f,-0.3f);
      gl.glVertex2f(0.5f,-0.3f);
      gl.glVertex2f(0,0.6f);
      gl.glEnd();
      gl.glColor3f(0,0,0);
      gl.glBegin(GL.GL_LINE_LOOP);
      gl.glVertex2f(-0.5f,-0.3f);
      gl.glVertex2f(0.5f,-0.3f);
      gl.glVertex2f(0,0.6f);
      gl.glEnd();
   }

   public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
   }

   public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
         boolean deviceChanged) {
   }
}
