package glutil;

import javax.media.opengl.GL;

/**
 * Represents a cube, centered at the origin, with sides
 * parallel to the coordinate axes.
 */
public class Cube implements Renderable {

   private final double side;  // length of a side

   /**
    * Create a cube that is 2 units on a side.
    */
   public Cube() {
      this(2);
   }

   /**
    * Create a cube with a given side length.
    */
   public Cube(double side) {
      this.side = side;
   }

   public void render(GL gl) {
      gl.glPushMatrix();
      gl.glRotatef(-90,-1,0,0);  // This puts the textures in the orientation I want.
      gl.glPushMatrix();
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);  // Each side of the cube is a transformed square.
      gl.glPopMatrix();
      gl.glPushMatrix();
      gl.glRotatef(90,0,1,0);
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);
      gl.glPopMatrix();
      gl.glPushMatrix();
      gl.glRotatef(180,0,1,0);
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);
      gl.glPopMatrix();
      gl.glPushMatrix();
      gl.glRotatef(270,0,1,0);
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);
      gl.glPopMatrix();
      gl.glPushMatrix();
      gl.glRotatef(90,-1,0,0);
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);
      gl.glPopMatrix();
      gl.glPushMatrix();
      gl.glRotatef(-90,-1,0,0);
      gl.glTranslated(0,0,side/2);
      square(gl,side/2);
      gl.glPopMatrix();
      gl.glPopMatrix();
   }

   /**
    * Draws a square in the xy-plane, with given radius,
    * where radius is half the length of the side.
    */
   private void square(GL gl, double radius) {
      gl.glBegin(GL.GL_POLYGON);
      gl.glNormal3f(0,0,1);
      gl.glTexCoord2d(0,0);
      gl.glVertex2d(-radius,-radius);
      gl.glTexCoord2d(1,0);
      gl.glVertex2d(radius,-radius);
      gl.glTexCoord2d(1,1);
      gl.glVertex2d(radius,radius);
      gl.glTexCoord2d(0,1);
      gl.glVertex2d(-radius,radius);
      gl.glEnd();
   }

}
