package scenegraph2D;

/**
 * A scene node representing a disk as a many-sided regular polygon.
 */
public class DiskNode extends PolygonNode {
   
   /**
    * Make a Disk of radius 1 with 32 sides.
    */
   public DiskNode() {
      this(1.0, 32);
   }

   /**
    * Make a Disk of specified radius, with 32 sides.
    * @param radius
    */
   public DiskNode(double radius) {
      this(radius, 32);
   }
   
   /**
    * Make a Disk of specified radius and number of sides.
    */
   public DiskNode(double radius, int sides) {
      super(makeVertices(radius, sides));
   }
   
   private static double[] makeVertices(double radius, int sides) {
      double[] coords = new double[2*sides];
      for (int i = 0; i < sides; i++) {
         double angle = (2*Math.PI/sides)*i;
         coords[2*i] = Math.cos(angle) * radius;
         coords[2*i+1] = Math.sin(angle) * radius;
      }
      return coords;
   }

}
