package scenegraph2D;

import java.awt.Color;

import javax.media.opengl.GL;

public abstract class SceneNode2D {
   
   private double scaleX = 1, scaleY  = 1;
   private double rotation = 0;
   private double translateX = 0, translateY = 0;
   
   private Color color;
   
   abstract protected void drawBasic(GL gl);
   
   public void draw(GL gl) {
      float[] currentColor = new float[4];
      gl.glGetFloatv(GL.GL_CURRENT_COLOR, currentColor, 0);
      gl.glPushMatrix();
      gl.glTranslated(translateX, translateY, 0);
      gl.glRotated(rotation, 0, 0, 1);
      gl.glScaled(scaleX, scaleY, 1);
      if (color != null) {
         float[] hsb = color.getColorComponents(null);
         gl.glColor3f(hsb[0], hsb[1], hsb[2]);
      }
      drawBasic(gl);
      gl.glPopMatrix();
      gl.glColor3f(currentColor[0], currentColor[1], currentColor[2]);
   }
   
   public void setScaling( double uniformScaleFactor ) {
      scaleX = scaleY = uniformScaleFactor;
   }
   
   public void setScaleing( double scaleX, double scaleY ) {
      this.scaleX = scaleX;
      this.scaleY = scaleY;
   }
   
   public void setTranslation( double dx, double dy ) {
      translateX = dx;
      translateY = dy;
   }

   public double getScaleX() {
      return scaleX;
   }

   public void setScaleX(double scaleX) {
      this.scaleX = scaleX;
   }

   public double getScaleY() {
      return scaleY;
   }

   public void setScaleY(double scaleY) {
      this.scaleY = scaleY;
   }

   public double getRotation() {
      return rotation;
   }

   public void setRotation(double rotation) {
      this.rotation = rotation;
   }

   public double getTranslateX() {
      return translateX;
   }

   public void setTranslateX(double translateX) {
      this.translateX = translateX;
   }

   public double getTranslateY() {
      return translateY;
   }

   public void setTranslateY(double translateY) {
      this.translateY = translateY;
   }

   public Color getColor() {
      return color;
   }

   public void setColor(Color color) {
      this.color = color;
   }
   

}
