package scenegraph2D;

import javax.media.opengl.GL;

/**
 * A convenience class for representing an object that is 
 * just a copy of another object, with a transform applied.
 * (Note that the same thing could be done with a ComplexObject2D
 * that contains just a single sub-object.)
 */
public class TransformedObject extends SceneNode2D {
   
   private SceneNode2D baseObject;
   
   protected void drawBasic(GL gl) {
      baseObject.draw(gl);
   }
   
   /**
    * Create a TransformedObject that wraps a given object.
    */
   public TransformedObject(SceneNode2D object) {
      baseObject = object;
   }
   
   /**
    * Create a TransformedObject that wraps a given object, with transformations.
    * @param object the object to be wrapped
    * @param scale the scaling that is to be applied to the object.
    * @param rotation the rotation that is to be applied to the object.
    * @param dx the translation in the x direction that is to be applied.
    * @param dy the translation in the y direction that is to be applied.
    */
   public TransformedObject(SceneNode2D object, double scale, double rotation, double dx, double dy) {
      baseObject = object;
      setScaling(scale);
      setRotation(rotation);
      setTranslation(dx, dy);
   }

}
