package scenegraph3d;

import java.util.ArrayList;

import javax.media.opengl.GL;

/**
 * A SceneNode3D that representing a complex object consisting of a list
 * of sub-objects that are themselves scene nodes.
 */
public class ComplexObject3D extends SceneNode3D {
   
   private ArrayList<SceneNode3D> subObjects = new ArrayList<SceneNode3D>();
   
   /**
    * Create an empty complex object.
    */
   public ComplexObject3D() {
   }
   
   /**
    * Create a complex object initially containing one object.
    * @param item to be added to the new complex object.  Cannot be null.
    */
   public ComplexObject3D(SceneNode3D item) {
      add(item);
   }

   protected void drawBasic(GL gl) {
      for (SceneNode3D item: subObjects) {
         item.draw(gl);
      }
   }
   
   public int turnOnLightsBasic(GL gl, int nextLightNumber) {  // turn on lights in all the subobjects
      for (SceneNode3D item : subObjects) {
         nextLightNumber = item.turnOnLights(gl,nextLightNumber);
      }
      return nextLightNumber;
   }
   
   /**
    * Add a sub-object to this complex object.  Returns a pointer
    * to this ComplexObject3D, for use in chainging of method calls.
    */
   public ComplexObject3D add(SceneNode3D item) {
      if (item == null)
         throw new IllegalArgumentException("Can't add a null to a CompleObject3D");
      subObjects.add(item);
      if (item.parents == null)
         item.parents = new ArrayList<SceneNode3D>();
      item.parents.add(this);
      return this;
   }
   
   /**
    * Returns the number of child nodes of this node.
    * @return
    */
   public int getItemCount() {
      return subObjects.size();
   }
   
   /**
    * Gets the child node at a given index in the list of child nodes.  Will
    * through an exception if the index is out of range.
    */
   public SceneNode3D getItem(int index) {
      return subObjects.get(index);
   }
   
   /**
    * Removes a given child node from this node, if it is present.  No
    * error occurs if the given node is not actually a child of this node.
    * @return true if the item was present and has been removed.
    */
   public boolean remove(SceneNode3D item) {
      if (subObjects.remove(item)) {
         item.parents.remove(this);
         if (item.parents.size() == 0)
            item.parents = null;
         return true;
      }
      return false;
   }
   
   /**
    * Removes the child node at a given index, if that index is
    * in range.  No error occurs if it is not.
    */
   public void remove(int index) {
      if (index >= 0 && index < subObjects.size()) {
         SceneNode3D item = subObjects.get(index);
         subObjects.remove(index);
         item.parents.remove(this);
         if (item.parents.size() == 0)
            item.parents = null;
      }
   }
   
      
}
