package selection;

import javax.media.opengl.GL;

import simplescenegraph3d.ComplexObject3D;
import simplescenegraph3d.SceneNode3D;

/**
 * Represents an object that is the same as a ComplexObject3D, except that as it
 * renders the items that it contains, it also manages the name stack that is used
 * for picking objects with GL_SELECT rendering mode.  The name associated with the
 * i-th sub-object of this object is the number i.  This name is placed on the
 * name stack while the object is being rendered, so that it will show up in
 * a hit record in select mode if it is under the mouse.
 */
public class ComplexObject3DWithSelectionNames extends ComplexObject3D {

   protected void drawBasic(GL gl) {
      int ct = getItemCount();
      if (ct == 0)
         return;
      gl.glPushName(Integer.MAX_VALUE); // Placeholder value will be replaced with item number by glLoadName.
      for (int i = 0; i < ct; i++) {
         gl.glLoadName(i);  // i is put on the name stack as the name for item number i in this complex object.
         SceneNode3D item = getItem(i);
         item.draw(gl);
      }
      gl.glPopName();  // Remove the item from the name stack.
   }

}
