package stereoGraph3d;


import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.media.opengl.*;


public class TestStereo extends JPanel implements GLEventListener {

   public static void main(String[] args) {
      JFrame window = new JFrame("Paddle Wheels");
      window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      window.setContentPane(new TestStereo());
      window.pack();
      window.setVisible(true);
   }
   
   private GLJPanel drawable;  // The panel where the drawing is done.
   
   private CameraWithStereo camera;  // Contains the current view transform and projection.
                           // The user can drag with the mouse to rotate the view.
   
   private JComboBox levelSelect;  // Select level of recursion for fractal.

   /**
    * Make a regular JPanel to hold the GLJPanel (although in this
    * program, that's all it holds.
    */
   public TestStereo() {
      drawable = new GLJPanel();
      drawable.setPreferredSize(new Dimension(600,600));
      drawable.addGLEventListener(this);
      setLayout(new BorderLayout());
      add(drawable, BorderLayout.CENTER);
      camera = new CameraWithStereo();
      camera.setLimits(-7, 7, -7, 7, -10, 10);
      camera.setView(0,0,60,0,0,0,0,1,0);
      new TrackBallForStereoCamera(drawable,camera);  
      levelSelect = new JComboBox();
      for (int i = 0; i < 8; i++)
         levelSelect.addItem( (i+1) + " levels of recursion ");
      levelSelect.setSelectedIndex(3);
      levelSelect.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            drawable.repaint();
         }
      });
      JPanel bottom = new JPanel();
      bottom.setLayout(new FlowLayout(FlowLayout.CENTER));
      bottom.add(levelSelect);
      add(bottom, BorderLayout.SOUTH);
   }

   /**
    * This method will be called when the GLJPanel is first
    * created.  It can be used to initialize the GL context.
    */
   public void init(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glClearColor(0,0,0,1);
      gl.glEnable(GL.GL_LIGHTING);
      gl.glEnable(GL.GL_LIGHT0);
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, new float[] {0.4f, 0.4f, 0.4f, 1}, 0);
      gl.glEnable(GL.GL_LIGHT1);
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, new float[] {1,1,1, 1}, 0);
      gl.glEnable(GL.GL_DEPTH_TEST);
      gl.glShadeModel(GL.GL_SMOOTH);
      //gl.glEnable(GL.GL_COLOR_MATERIAL);
      gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE, new float[]{1,1,1,1}, 0);
      gl.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, new float[] {0,0,0,0}, 0);
      //gl.glLightModeli(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
   }

   /**
    * Draw four spheres, with different settings. The spheres are
    * actually polyhedra with a rather small number of faces, not 
    * enough to give something that looks exactly like a sphere, 
    * even with lighting and smooth shading.
    */
   public void display(GLAutoDrawable drawable) {
      GL gl = drawable.getGL();
      gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
      
      camera.apply(gl, -3);
      gl.glColorMask(true,false,false,false);
      
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, new float[] {0, 0, 0, 1}, 0);

      
      drawTetrahedron(gl, levelSelect.getSelectedIndex() + 1,
                  new double[] {0,4,0}, new double[] {-4,-3,-4}, new double[] {0,-3, 3}, new double[] {4,-3,-4},
                  new double[] {0.5,0.5,0.5}, new double[] {1,0,0}, new double[] {0,1,0}, new double[] {0,0,1});

      gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
      camera.apply(gl, 3);
      gl.glColorMask(false,true,false,false);
      
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, new float[] {0, 0, 0, 1}, 0);

      
      drawTetrahedron(gl, levelSelect.getSelectedIndex() + 1,
                  new double[] {0,4,0}, new double[] {-4,-3,-4}, new double[] {0,-3, 3}, new double[] {4,-3,-4},
                  new double[] {0.5,0.5,0.5}, new double[] {1,0,0}, new double[] {0,1,0}, new double[] {0,0,1});

      gl.glColorMask(true,true,true,true);
   }
   
   private static void drawTetrahedron(GL gl, int level, double A[], double[] B, double[] C, double D[],
                                double[] cA, double[] cB, double[] cC, double[] cD) {
      if (level <= 1) {
           drawTriangle(gl,A,B,C,cA,cB,cC);
           drawTriangle(gl,A,C,D,cA,cC,cD);
           drawTriangle(gl,A,D,B,cA,cD,cB);
           drawTriangle(gl,B,D,C,cB,cD,cC);
      }
      else {
         double[] AB2 = new double[] { (A[0]+B[0])/2, (A[1]+B[1])/2, (A[2]+B[2])/2 };
         double[] AC2 = new double[] { (A[0]+C[0])/2, (A[1]+C[1])/2, (A[2]+C[2])/2 };
         double[] AD2 = new double[] { (A[0]+D[0])/2, (A[1]+D[1])/2, (A[2]+D[2])/2 };
         double[] BC2 = new double[] { (B[0]+C[0])/2, (B[1]+C[1])/2, (B[2]+C[2])/2 };
         double[] BD2 = new double[] { (B[0]+D[0])/2, (B[1]+D[1])/2, (B[2]+D[2])/2 };
         double[] CD2 = new double[] { (C[0]+D[0])/2, (C[1]+D[1])/2, (C[2]+D[2])/2 };
         double[] cAB2 = new double[] { (cA[0]+cB[0])/2, (cA[1]+cB[1])/2, (cA[2]+cB[2])/2 };
         double[] cAC2 = new double[] { (cA[0]+cC[0])/2, (cA[1]+cC[1])/2, (cA[2]+cC[2])/2 };
         double[] cAD2 = new double[] { (cA[0]+cD[0])/2, (cA[1]+cD[1])/2, (cA[2]+cD[2])/2 };
         double[] cBC2 = new double[] { (cB[0]+cC[0])/2, (cB[1]+cC[1])/2, (cB[2]+cC[2])/2 };
         double[] cBD2 = new double[] { (cB[0]+cD[0])/2, (cB[1]+cD[1])/2, (cB[2]+cD[2])/2 };
         double[] cCD2 = new double[] { (cC[0]+cD[0])/2, (cC[1]+cD[1])/2, (cC[2]+cD[2])/2 };
         drawTetrahedron(gl, level-1, A, AB2, AC2, AD2, cA, cAB2, cAC2, cAD2);
         drawTetrahedron(gl, level-1, B, AB2, BD2, BC2, cB, cAB2, cBD2, cBC2);
         drawTetrahedron(gl, level-1, C, AC2, BC2, CD2, cC, cAC2, cBC2, cCD2);
         drawTetrahedron(gl, level-1, D, CD2, BD2, AD2, cD, cCD2, cBD2, cAD2);
      }
   }
   
   private static void drawTriangle(GL gl, double vertexA[], double[] vertexB, double[] vertexC,
                           double[] colorA, double[] colorB, double[] colorC ) {
      double[] v1 = new double[] {vertexB[0]-vertexA[0], vertexB[1]-vertexA[1], vertexB[2]-vertexA[2]};
      double[] v2 = new double[] {vertexC[0]-vertexA[0], vertexC[1]-vertexA[1], vertexC[2]-vertexA[2]};
      double[] perp = new double[] { v1[1]*v2[2]-v2[1]*v1[2], v1[2]*v2[0]-v2[2]*v1[0], v1[0]*v2[1]-v1[1]*v2[0]};
      double norm = Math.sqrt( perp[0]*perp[0] + perp[1]*perp[1] + perp[2]*perp[2] );
      gl.glNormal3d(perp[0]/norm, perp[1]/norm, perp[2]/norm);
      gl.glBegin(GL.GL_POLYGON);
      gl.glColor3dv(colorA,0);
      gl.glVertex3dv(vertexA,0);
      gl.glColor3dv(colorB,0);
      gl.glVertex3dv(vertexB,0);
      gl.glColor3dv(colorC,0);
      gl.glVertex3dv(vertexC,0);
      gl.glEnd();
   }
   
   
   public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
   }

   public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
         boolean deviceChanged) {
   }
}
